﻿using System;
using System.Collections.Generic;

namespace MethodOverriding
{
    class A
    {
        public virtual A Method()
        {
            Console.WriteLine("Class A, Method");
            return new A();
        }

        public virtual ICollection<String> GetCollection(String str)
        {
            Console.WriteLine("Class A, GetCollection");
            return new HashSet<string> {str};
        }

        public virtual void Method2(object str)
        {
            Console.WriteLine(str);
        }

        public override string ToString()
        {
            return "A object";
        }
    }

    class B : A
    {
        // Overriding: Same name, same parameters, 
        //             return type is the same or a subtype
        public override A Method()
        {
            return new B();
        }

        public new IList<string> GetCollection(string str)
        {
            Console.WriteLine("Class B, GetCollection");
            return new List<string> {str};
        }

        // overriding: Same name, different parameter list
        public override sealed void Method2(object str)
        {
            Console.WriteLine(str);
        }

        // method overriding same parameter (none) and same return type (string)
        public override string ToString()
        {
            return "B object";
        }

        public void SpecialMethod() { }
    }

    class C : B
    {
        // cannot override, because Method2 is sealed in the base class
        //public override void Method2(object st) 
        //{
            
        //}
    }

    class Program
    {
        static void Main()
        {
            A a = new A();
            Console.WriteLine(a.ToString());
            A a2 = new B();
            Console.WriteLine(a2.ToString());
            //a2.SpecialMethod()
            B b = new B();
            Console.WriteLine(b.ToString());

            A another = a.Method();
            Console.WriteLine(another.ToString());

            A another2 = a2.Method();
            Console.WriteLine(another2);

            ICollection<String> coll = a2.GetCollection("hello");
            Console.WriteLine(String.Join(", ", coll));
        }
    }
}
